/* 
 * $Id: ExcludedEventService.java 4282 2013-06-12 17:44:50Z mitcje $
 * $URL: https://open.jira.com/svn/IKASAN/tags/ikasaneip-0.9.0/spec/service/error/src/main/java/org/ikasan/spec/exclusion/ExcludedEventService.java $ 
 *
 * ====================================================================
 * Ikasan Enterprise Integration Platform
 * 
 * Distributed under the Modified BSD License.
 * Copyright notice: The copyright for this software and a full listing 
 * of individual contributors are as shown in the packaged copyright.txt 
 * file. 
 * 
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 *
 *  - Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *
 *  - Neither the name of the ORGANIZATION nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 */
package org.ikasan.spec.exclusion;

import org.ikasan.spec.exclusion.ExcludedEvent;

/**
 * @author The Ikasan Development Team
 * 
 */
public interface ExcludedEventService<EVENT,PAGEDRESULT>
{

    /**
     * Exclude and Event from a specified flow
     * 
     * @param event
     * @param moduleName
     * @param flowName
     */
    public void excludeEvent(EVENT event, String moduleName, String flowName);

    /**
     * Returns a paged listing of ExcludedEvent
     * 
     * @param pageNo - 0 or greater, index into the list of all possible results
     * @param pageSize - 0 or greater, no of excludedEvents to return on a page
     * @param orderBy - field to order by
     * @param orderAscending - in ascending order?
     * @param flowName - restrict by flowName if supplied
     * @param moduleName - restrict by moduleName if supplied
     * 
     * @return PagedSearchResult<ExcludedEvent>
     */
    public PAGEDRESULT getExcludedEvents(int pageNo, int pageSize, String orderBy, boolean orderAscending, String moduleName, String flowName);

    /**
     * Retrieve an ExcludedEvent specified by its event Id
     * 
     * @param eventId
     * @return ExcludedEvent
     */
    public ExcludedEvent getExcludedEvent(String eventId);

    /**
     * Synchronously attempts to resubmit an ExcludedEvent specified by id
     * 
     * 
     * @throws IllegalArgumentException if ExcludedEvent cannot be found, or if
     *             referenced module or flow are not available
     * 
     * @param eventId
     * @param resubmitter
     */
    public void resubmit(String eventId, String resubmitter);

    /**
     * attempts to resolve an ExcludedEvent specified by id as cancelled
     * 
     * 
     * @throws IllegalArgumentException if ExcludedEvent cannot be found, or if
     *             referenced module or flow are not available
     * 
     * @param eventId
     * @param canceller
     */
    public void cancel(String eventId, String canceller);

}
